#!/usr/bin/env python
"""ClientAPI.py
Maintained by W.Yang

An interface layer for application to connect database handler.
The following API is provided:

OpenDB(host,username,password,dbname)
CloseDB()
IsOpen()
NewProject(projectname)
NewJob(projectname,jobname)
GetJobid(projectname,jobname)
GetJobname(jobid)
SetData(jobid,name,value)
GetData(jobid,name)
UpdateData(factid,value)
ImportDingbat(jobid,name,description,type,value)
ExportDingbat(jobid,name)
DescribeDingbat(jobid,name)
DeleteDingbat(jobid,name)
ListProjects()
ListJobs(projectname)
ListItems(jobid)
GetProject(projectname)
DeleteProject(projectname)
Version()

OpenDB(host,username,password,dbname) should be called first before any other methods can be used. When finish all the operations, CloseDB() should be called to close the connection.

"""
#CVS Id $Id: ClientAPI.py,v 1.30 2005/10/07 12:28:55 wy45 Exp $

import socket
import base64
import sys
import xml.dom.minidom
import xml.sax.saxutils as su
import exceptions
import thread
from xml.dom.ext.reader import Sax2


class serverconnection:
   #set the socket parameters
   
   def __init__(self):
      self.host = 'localhost'
      self.port = 8883
      self.buf = 40960
      self.addr = (self.host,self.port)

      self.sock = None
      self.no = None

      self.lock = thread.allocate_lock()
      
     #create a socket
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

     #Connect to server
      self.sock.connect((self.host,self.port))
   def __del__(self):
      pass      

   def CloseDB(self):

      """ Close database. """

      data = "<db_request><command>CloseDB</command></db_request>"

      #send end of connection message to host
      self.lock.acquire(True)

      try:
         self.sock.send(data)
         response = self.sock.recv(40960)
     
         #close the socket
         self.sock.close()
      
      except:
         self.lock.release()
         raise
      self.lock.release()

      parse_response(response)
         
   def OpenDB(self,host,username,password,dbname):
      """  Open a specific database.  """

      if not host:
         raise ValueError, "OpenDB(host,username.password,dbname): No host name"
      if not username:
         raise ValueError, "OpenDB(host,username.password,dbname): No username"
      if not password:
         raise ValueError, "OpenDB(host,username.password,dbname): No password"
      if not dbname:
         raise ValueError,"OpenDB(host,username.password,dbname): No dbname"
      
      #get command name and parameter
      argu_list = [host,username,password,dbname]
      data = request_wrapper('OpenDB',argu_list)

      self.lock.acquire(True)
      try:

         #send a request to the host
         self.sock.send(data)
      
         #get the response
         response = self.sock.recv(40960)

      except:
          self.lock.release()
          raise
      self.lock.release()
      
      parse_response(response)

   def IsOpen(self):
      ''' Check whether database open or not'''
      
      data = "<db_request><command>IsOpen</command></db_request>"
      self.lock.acquire(True)
      try:
         self.sock.send(data)
         response = self.sock.recv(40960)
      except:
         self.lock.release()
         raise
      self.lock.release()
        
      final = parse_response(response)
      if final[0]=="True":
         result=True
      elif final[0]=="False":
         result=False
      else:
         raise RuntimeError, "ClientAPI.IsOpen received %s." % final[0]
      
      return result
   
   def NewProject(self,projectname):
       """ Create a new project. Return project id. """
       #verify the parameter
       if not projectname:
          raise ValueError, "NewProject(projectname): no project name"

       else:
          #get command name and parameter
          argu_list = [projectname]
          data = request_wrapper('NewProject',argu_list)


          self.lock.acquire(True)
          try:
             #Send a request to the host
             self.sock.send(data)

             # Get the host response
             res = self.sock.recv(40960)
          except:
             self.lock.release()
             raise
          self.lock.release()
          
          final = parse_response(res)
          return int(final[0])
          
   def NewJob(self,projectname,jobname):
       """  Create a new job within the specified project with specified name,
            Return job id."""
       # verify the parameter
       if not projectname: 
          raise ValueError, "NewJob(projectname,jobname): no projectname"
       if not jobname:
          raise ValueError, "NewJob(projectname,jobname): no jobname"

       else:
          #get command name and parameter
          argu_list = [projectname,jobname]
          data = request_wrapper('NewJob',argu_list)


          self.lock.acquire(True)
          try:
             #Send a request to the host
             self.sock.send(data)

             # Get the host's response
             res = self.sock.recv(40960)
          except:
             self.lock.release()
             raise
          self.lock.release()
          
          final = parse_response(res)
          return int(final[0])

   def SetData(self,jobid,name,value):
       """   Create an item within the specified job. Return fact id ."""
       # check data
       if not jobid:
          raise ValueError, "SetData(jobid,name,value): no jobid"
       if not name:
          raise ValueError, "SetData(jobid,name,value): no name"
       if not value:
          raise ValueError, "SetData(jobid,name,value): no value"
    
       if not type(value) in [type(1), type('a'),type(0.1)]:
          raise ValueError, "Value can only be int, string or float in SetData"

       try:
          jobid = int(jobid)
       except:
          raise ValueError, "SetData(jobid,name,value): jobid should be integer"

       #get command name and parameter
       argu_list = [jobid,name,value]
       data = request_wrapper('SetData',argu_list)
            
       self.lock.acquire(True)
       try:
           #Send a request to the host
           self.sock.send(data)

           # Get the host's response
           response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()
       
       final = parse_response(response)
       return int(final[0])
   
   def GetData(self,jobid,name):
       """  Return the latest value of specified item name within specifed job. """
       if not jobid:
          raise ValueError, "GetData(jobid,name): no jobid"
       if not name:
          raise ValueError, "GetData(jobid,name): no name"

       try:
          jobid = int(jobid)
       except:
          raise ValueError,"GetData(jobid,name): jobid should be integer"
       
       # get command name and parameter
       argu_list = [jobid,name]
       data = request_wrapper('GetData',argu_list)

       self.lock.acquire(True)
       try:
          
          #Send a request to the host
          self.sock.send(data)

          # Get the host's response
          response = self.sock.recv(40960)
       except:
           self.lock.release()
           raise
       self.lock.release()

       final = parse_response(response)
       return final[0]
   
   def ListProjects(self):
       """  Return list of projects ."""     
       #get command name and parameter
       data = "<db_request><command>ListProjects</command></db_request>"


       self.lock.acquire(True)
       try:
          #Send a request to the host
          self.sock.send(data)

          # Get the host's response
          response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       
       self.lock.release()
   
       parse_response(response)
       final = parse_response(response)
       return final

   def ListJobs(self,projectname):
       """  Return a list of jobs within specified projects. """       
       if not projectname:
          raise ValueError, "ListJobs(projectname): no project name"

       #get command name and parameter
       argu_list = [projectname]
       data = request_wrapper('ListJobs',argu_list)


       self.lock.acquire(True)
       try:

          #Send a request to the host
          self.sock.send(data)

          # Get the host's response
          response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()

       final = parse_response(response)
       return final  
 
   def ListItems(self,jobid):
       """  Return a list of items within specified job. """
       if not jobid:
          raise ValueError, "ListItems(jobid): no jobid"
       try:
          jobid = int(jobid)
       except:
          raise ValueError, "ListItems(jobid): jobid should be integer"
       
       #get command name and parameter
       argu_list = [jobid]
       data = request_wrapper('ListItems',argu_list)

       self.lock.acquire(True)
       try:

          #Send a request to the host
          self.sock.send(data)

          # Get the host's response
          response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()
       
       final = parse_response(response)
       return final
 
   def ImportDingbat(self,jobid,name,description,type,value):
       """ Create a new dingbat for a specified job. The dingbat will have
     name,description,type and value attributes. Return dingbat id """
       if not jobid:
          raise ValueError, "ImportDingbat(jobid,name,description,type,value): no jobid"
       if not name:
           raise ValueError, "ImportDingbat(jobid,name,description,type,value): no name"
       if not description:
          raise ValueError, "ImportDingbat(jobid,name,description,type,value): no description"
       if not type:
           raise ValueError, "ImportDingbat(jobid,name,description,type,value): no type"
       if not value:
          raise ValueError, "ImportDingbat(jobid,name,description,type,value): no value"
       try:
          jobid = int(jobid)
       except:
           raise ValueError, "ImportDingbat(jobid,name,description,type,value):jobid should be integer"
       
       #get command name and parameter
       encoded_value = base64.encodestring(value).replace('\n','')
       argu_list = [jobid,name,description,type,encoded_value]
       data = request_wrapper('ImportDingbat',argu_list)

       self.lock.acquire(True)
       try:
          #Send a request to the host
          self.sock.send(data)

          # Get the host's response
          response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()
       
       final = parse_response(response)

       return final
 
   def ExportDingbat(self,jobid,name):
       """Return the latest value associated with the most recent dingbat stored in a specific job and specific name. """  

       if not jobid:
          raise ValueError, "ExportDingbat(jobid,name): no jobid"
       if not name:
          raise ValueError, "ExportDingbat(jobid,name): no name"

       try:
          jobid = int(jobid)
       except:
          raise ValueError, "ExportDingbat(jobid,name): jobid should be integer"
       
       #get command name and parameter
       argu_list = [jobid,name]
       data = request_wrapper('ExportDingbat',argu_list)


       self.lock.acquire(True)
       try:
          #Send a request to the host
          self.sock.send(data)

          # Get the host's response
          response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()

       value = parse_response(response)
       final = base64.decodestring(value[0])
               
       return final
    
   def DeleteDingbat(self,jobid,name):
       """  Remove a dingbat and its associated data from a specific project for            a specific job and specific name. """      
       if not jobid:
          raise ValueError, "DeleteDingbat(jobid,name): no jobid"
       if not name:
          raise ValueError,"DeleteDingbat(jobid,name): no name"
       try:
          jobid = int(jobid)
       except:
          raise ValueError,"DeleteDingbat(jobid,name): jobid should be integer"

       #get command name and parameter
       argu_list = [jobid,name]
       data = request_wrapper('DeleteDingbat',argu_list)  


       self.lock.acquire(True)
       try:
           #Send a request to the host
           self.sock.send(data)

           # Get the host's response
           response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()
       
       parse_response(response)

   def DescribeDingbat(self,jobid,name):
       """ Return the description and type attributes of a dingbat for a specified job. """   
       if not jobid:
          raise ValueError,"DescribeDingbat(jobid,name): no jobid"
       if not name:
          raise ValueError,"DescribeDingbat(jobid,name): no name"
       try:
          jobid = int(jobid)
       except:
          raise ValueError, "DescribeDingbat(jobid,name): jobid should be integer"

       #get command name and parameter
       argu_list = [jobid,name]
       data = request_wrapper('DescribeDingbat',argu_list)  
           
       self.lock.acquire(True)
       try:
          
           #Send a request to the host
           self.sock.send(data)

           # Get the host's response
           response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()
    
       final = parse_response(response)
       return final 

 
   def GetProject(self,projectname):
      """  Open an exsiting project. Return project id.  """

      if not projectname:
         raise ValueError, "GetProject(projectname): no projectname"
      
      #get command name and parameter
      argu_list = [projectname]
      data = request_wrapper('GetProject',argu_list)  

      self.lock.acquire(True)
      try:
         #send a request to the post
         self.sock.send(data)

         #get the host's response
         response = self.sock.recv(40960)
      except:
         self.lock.release()
         raise
      self.lock.release()
      
      final = parse_response(response)
      return int(final[0])
 
   def DeleteProject(self,projectname):
      ''' Delete an exsiting project. '''
      if not projectname:
         raise ValueError, "DeleteProject(projectname): no projectname"
      
      #get command name and parameter
      argu_list = [projectname]
      data = request_wrapper('DeleteProject',argu_list)  


      self.lock.acquire(True)
      try:
          #send a request to the post
          self.sock.send(data)

          #get the host's response
          response = self.sock.recv(40960)
      except:
         self.lock.release()
         raise
      self.lock.release()
      
      parse_response(response)
      
   def Version(self):
      """ Return version of MySQLdb package.  """
      #get command name and parameter
      data = "<db_request><command>Version</command></db_request>"
      self.lock.acquire(True)
      try:
          #send a reques to the post
          self.sock.send(data)

          #get the host's response
          response = self.sock.recv(40960)
      except:
         self.lock.release()
         raise
      self.lock.release()
      
      final = parse_response(response)
      return final  

   def UpdateData(self,factid,value):
       ''' Update the value of specified fact '''
       if not factid:
          raise ValueError, "UpdateData(factid,value): no factid"
       if not value:
          raise ValueError, "UpdateData(factid,value): no value"
       try:
          factid = int(factid)
       except:
          raise ValueError,"UpdateData(factid,value): factid should be integer"  
       argu_list = [factid,value]
       data = request_wrapper('UpdateData',argu_list)

       self.lock.acquire(True)
       try:
          self.sock.send(data)
          response = self.sock.recv(40960)
       except:
          self.lock.release()
          raise
       self.lock.release()
       
       parse_response(response)

   def GetJobid(self,projectname,jobname):
      ''' Return jobid given a project name and job name '''
      if not projectname:
         raise ValueError,"GetJobid(projectname,jobname): no projectname"
      if not jobname:
         raise ValueError,"GetJobid(projectname,jobname): no jobname"

      argu_list = [projectname,jobname]
      data = request_wrapper('GetJobid',argu_list)

      self.lock.acquire(True)
      try:
         self.sock.send(data)
         response = self.sock.recv(40960)
      except:
         self.lock.release()
         raise
      self.lock.release()
      
      final = parse_response(response)
      return int(final[0])

   def GetJobname(self,jobid):
      ''' Return jobname given a jobid'''
      if not jobid:
         raise ValueError,"GetJobname(jobid): no jobid"

      try:
         jobid = int(jobid)
      
      except:
          raise ValueError,"UpdateData(factid,value): factid should be integer"  
      argu_list = [jobid]
      data = request_wrapper('GetJobname',argu_list)

      self.lock.acquire(True)
      try:
         self.sock.send(data)
         response = self.sock.recv(40960)
      except:
         self.lock.release()
         raise
      self.lock.release()
      
      final = parse_response(response)
      return final[0]
      
 
def request_wrapper(command,arg_list):
   data = "<db_request><command>"+command+"</command>"
   for arg in arg_list:
      data = data + "<argument>"+su.escape(str(arg))+"</argument>"
   data = data+"</db_request>"

   return data

def parse_response(xml_response):
    ''' Parse response '''
    resultlist = []

    reader = Sax2.Reader()
    
    doc = reader.fromString(xml_response)
#    doc = xml.dom.minidom.parseString(xml_response)
    
    statusTag = doc.getElementsByTagName("status")
    status = statusTag[0].childNodes[0].data

    resultTag = doc.getElementsByTagName("result")
    for result in resultTag:
       a = result.childNodes[0].data
       resultlist.append(str(a))

    if status=="failed":
       raise RuntimeError, resultlist[0]

    return resultlist
 
